#--
# Copyright (c) 1996-2004, Yauheni Akhotnikau
# Copyright (c) 2004-2006, JSC Intervale
# Copyright (c) 2006-2015, The Mxx_ru Project
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#++

require 'mxx_ru/cpp/target'

module  MxxRu

  module Cpp

    # Target type is a composition of subprojects
    class CompositeTargetType < TargetType
      TYPE = "composite"

      def name
        return TYPE
      end
    end

    # For compatibility with previous versions.
    Composite_target_type = CompositeTargetType

    # Composition of subprojects target
    class CompositeTarget < Target
      def initialize( a_alias, &block )
        super( a_alias )

        instance_eval( &block ) if block
      end

      def target_type
        return CompositeTargetType.new
      end

      def build
        if nil == @mxx_last_build_result

          check_all_options_definition

          @mxx_last_build_result = build_required_projects
        end

        return @mxx_last_build_result

      end

      def clean
        if nil == @mxx_last_build_result

          check_all_options_definition

          clean_required_prjs

          @mxx_last_build_result = MxxRu::TargetState.new(
            MxxRu::TargetState::ABSENT )
        end
      end

      def reset
        reset_required_projects
        @mxx_last_build_result = nil
      end

    end # class CompositeTarget

    # For compatibility with previous versions.
    Composite_target = CompositeTarget

    # Simple method to define composite target.
    #
    # Example:
    #   MxxRu::Cpp::composite_target( MxxRu::BUILD_ROOT ) {
    #     required_prj 'some_project.rb'
    #     required_prj 'some_another_project.rb'
    #   }
    def Cpp.composite_target( prj_alias = nil, &block )
      prj_alias = MxxRu::Util::prj_alias_form_caller( caller ) unless prj_alias
      MxxRu::setup_target CompositeTarget.new( prj_alias, &block )
    end

  end # module Cpp

end # module MxxRu

