#--
# Copyright (c) 1996-2004, Yauheni Akhotnikau
# Copyright (c) 2004-2006, JSC Intervale
# Copyright (c) 2006-2015, The Mxx_ru Project
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#++

require 'mxx_ru/cpp/composite'
require 'mxx_ru/cpp/toolset'

module MxxRu
module Cpp

# Target type is a collection of all required libraries from subprojects.
#
# Since v.1.4.0
class LibCollectionTargetType < TargetType
  TYPE = "lib_collection"

  def name
    return TYPE
  end
end

# Collection of all required libraries form subprojects.
#
# Since v.1.4.0
class LibCollectionTarget < CompositeTarget
  def initialize( a_alias, &block )
    super( a_alias, &block )
  end

  def target_type
    return LibCollectionTargetType.new
  end

  def build
    if nil == @mxx_last_build_result
      # Let base class makes all necessary building actions.
      super

      # We must collect names of all required libraries from
      # all subprojects.
      mxx_required_prjs.each do |d|
        if Toolset::has_linkable_dependecies?( d )
          mxx_add_required_libs( d.mxx_required_libs )
          mxx_add_required_lib_paths( d.mxx_required_lib_paths )
        end
      end
    end

    @mxx_last_build_result
  end

end # class LibCollectionTarget

# Simple method to define libraries collection target.
#
# Example:
#   MxxRu::Cpp::lib_collection_target {
#     required_prj 'some_project.rb'
#     required_prj 'some_another_project.rb'
#   }
#
# Since v.1.4.0
def Cpp.lib_collection_target( prj_alias = nil, &block )
  prj_alias = MxxRu::Util::prj_alias_form_caller( caller ) unless prj_alias
  MxxRu::setup_target LibCollectionTarget.new( prj_alias, &block )
end

end # module Cpp
end # module MxxRu

