#--
# Copyright (c) 1996-2004, Yauheni Akhotnikau
# Copyright (c) 2004-2006, JSC Intervale
# Copyright (c) 2006-2015, The Mxx_ru Project
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#++

require 'pathname'
require 'rexml/document'
require 'set'

require 'mxx_ru/cpp/qt4'

module MxxRu
module Cpp

begin
  IO.popen( 'pkg-config --version' ) do |file|
    # A special modification for the case when configuration of Qt4
    # is detected via 'pkg-config' command. For example on Debian Linux.
    # (This case is detected and used by Mikhail Lyossin).
    #
    # Method use_module is redefined to make two calls to pkg-config:
    # - the first call with argument '--cflags' to detect include path and
    #   necessary defines;
    # - the second call with argument '--libs' to detect libraries path and
    #   names of necessry libraries.
    #
    class Qt4
      def use_module( a_module )
        @target.define a_module.define
        
        IO.popen( "pkg-config #{a_module.library} --cflags" ) { |file|
          @target.compiler_option( file.gets.chop )
        }
        IO.popen( "pkg-config #{a_module.library} --libs" ) { |file|
          @target.linker_option( file.gets.chop )
        }
      end
    end
  end
rescue 
end

end # module Cpp
end # module MxxRu

