#--
# Copyright (c) 1996-2004, Yauheni Akhotnikau
# Copyright (c) 2004-2006, JSC Intervale
# Copyright (c) 2006-2015, The Mxx_ru Project
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#++

require 'mxx_ru/cpp/toolsets/gcc_unix_family'

module  MxxRu
  module  Cpp
    module Toolsets

      # Toolset implemetation for GCC compiler for MacOS X (Darwin).
      class GccDarwin < GccFamily
      public
        def initialize( a_name = "gcc" )
          super( a_name )

          setup_tag( GCC_PORT_TAG, GCC_PORT_UNIX )
          setup_tag( "host_os", "unix" )
          setup_tag( "target_os", "unix" )
          setup_tag( "unix_port", "darwin" )
        end

        def setup_mandatory_options( target )
          super( target )

          if target.target_type.name == DllTargetType::TYPE
            target.linker_option( "-prebind" )
            target.linker_option( "-dynamiclib" )
            target.linker_option( "-single_module" )
          elsif target.target_type.name == MacOSBundleTargetType::TYPE
            target.linker_option( "-prebind" )
            target.linker_option( "-bundle" )
            target.linker_option( "-single_module" )
          elsif target.target_type.name == ExeTargetType::TYPE
            target.linker_option( "-prebind" )
          end
        end

        # There ins't any bracket on MacOS X.
        def enclose_linker_include_lib_options_into_brackes( options )
          options
        end

        # See description at MxxRu::Cpp::Toolset#dll_file_name.
        #
        # If we are building bundle then extension should be '.bundle'
        def dll_file_name( source_name, target )
          if target.target_type.name == MacOSBundleTargetType::TYPE
            construct_target_name( source_name, 'lib', '.bundle', target )
          else
            super
          end
        end
      end # class GccDarwin

    end # module Toolsets

  end # module Cpp

end # module MxxRu

MxxRu::Cpp::setup_toolset( MxxRu::Cpp::Toolsets::GccDarwin.new )

