#--
# Copyright (c) 1996-2004, Yauheni Akhotnikau
# Copyright (c) 2004-2006, JSC Intervale
# Copyright (c) 2006-2015, The Mxx_ru Project
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#++

require 'mxx_ru/cpp/toolsets/gcc_unix_family'

module  MxxRu
  module  Cpp
    module Toolsets

      # Toolset implemetation for GCC port on SPARC Solaris.
      class GccSparcSolaris < GccUnixFamily
      public
        def initialize( a_name = "gcc" )
          super( a_name )

          setup_tag( GCC_PORT_TAG, GCC_PORT_UNIX )
          setup_tag( "host_os", "unix" )
          setup_tag( "target_os", "unix" )
          setup_tag( "unix_port", "solaris" )
          setup_tag( "arch", "sparc" )
        end

        def setup_mandatory_options( target )
          super( target )

          target.linker_option( "-L/usr/lib" )
        end

        # See description at
        #MxxRu::Cpp::Toolsets::GccFamily#port_specific_dll_link_options.
        #
        # -mimpure-text linker option is added.
        def port_specific_dll_link_options(
          a_dll_name, a_dll_info, a_linker_lists, a_target )

          r = super( a_dll_name, a_dll_info, a_linker_lists, a_target ) +
            " -mimpure-text"

          return r
        end

        # Return string containing port-specific linker option for EXE linking.
        #
        # -mimpure-text linker option is added.
        def port_specific_exe_link_options(
          a_exe_name, a_exe_info, a_linker_lists, a_target )

          r = super( a_exe_name, a_exe_info, a_linker_lists, a_target ) +
            " -mimpure-text"

          return r
        end

      end # class GccSparcSolaris

      # For compatibility with previous versions.
      Gcc_sparc_solaris = GccSparcSolaris

    end # module Toolsets

  end # module Cpp

end # module MxxRu

MxxRu::Cpp::setup_toolset( MxxRu::Cpp::Toolsets::GccSparcSolaris.new )

